"""
https://github.github.com/gfm/#paragraphs
"""
import pytest

from .utils import act_and_assert


@pytest.mark.gfm
def test_paragraph_blocks_189():
    """
    Test case 189:  simple case of paragraphs
    """

    # Arrange
    source_markdown = """aaa

bbb"""
    expected_tokens = [
        "[para(1,1):]",
        "[text(1,1):aaa:]",
        "[end-para:::True]",
        "[BLANK(2,1):]",
        "[para(3,1):]",
        "[text(3,1):bbb:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>aaa</p>
<p>bbb</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_blocks_190():
    """
    Test case 189:  Paragraphs can contain multiple lines, but no blank lines:
    """

    # Arrange
    source_markdown = """aaa
bbb

ccc
ddd"""
    expected_tokens = [
        "[para(1,1):\n]",
        "[text(1,1):aaa\nbbb::\n]",
        "[end-para:::True]",
        "[BLANK(3,1):]",
        "[para(4,1):\n]",
        "[text(4,1):ccc\nddd::\n]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>aaa
bbb</p>
<p>ccc
ddd</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_blocks_190a():
    """
    Test case 189:  Paragraphs can contain multiple lines, but no blank lines:
    """

    # Arrange
    source_markdown = """aaa
bbb
ccc

ddd
eee
fff"""
    expected_tokens = [
        "[para(1,1):\n\n]",
        "[text(1,1):aaa\nbbb\nccc::\n\n]",
        "[end-para:::True]",
        "[BLANK(4,1):]",
        "[para(5,1):\n\n]",
        "[text(5,1):ddd\neee\nfff::\n\n]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>aaa
bbb
ccc</p>
<p>ddd
eee
fff</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_blocks_191():
    """
    Test case 191:  Multiple blank lines between paragraph have no effect:
    """

    # Arrange
    source_markdown = """aaa


bbb"""
    expected_tokens = [
        "[para(1,1):]",
        "[text(1,1):aaa:]",
        "[end-para:::True]",
        "[BLANK(2,1):]",
        "[BLANK(3,1):]",
        "[para(4,1):]",
        "[text(4,1):bbb:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>aaa</p>
<p>bbb</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_blocks_192():
    """
    Test case 192:  Leading spaces are skipped:
    """

    # Arrange
    source_markdown = """  aaa
 bbb"""
    expected_tokens = [
        "[para(1,3):  \n ]",
        "[text(1,3):aaa\nbbb::\n]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>aaa
bbb</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_blocks_193():
    """
    Test case 193:  Lines after the first may be indented any amount, since indented code blocks cannot interrupt paragraphs.
    """

    # Arrange
    source_markdown = """aaa
             bbb
                                       ccc"""
    expected_tokens = [
        "[para(1,1):\n             \n                                       ]",
        "[text(1,1):aaa\nbbb\nccc::\n\n]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>aaa
bbb
ccc</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_blocks_194():
    """
    Test case 194: (part a) However, the first line may be indented at most three spaces, or an indented code block will be triggered:
    """

    # Arrange
    source_markdown = """   aaa
bbb"""
    expected_tokens = [
        "[para(1,4):   \n]",
        "[text(1,4):aaa\nbbb::\n]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>aaa
bbb</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_blocks_195():
    """
    Test case 195:  (part b) However, the first line may be indented at most three spaces, or an indented code block will be triggered:
    """

    # Arrange
    source_markdown = """    aaa
bbb"""
    expected_tokens = [
        "[icode-block(1,5):    :]",
        "[text(1,5):aaa:]",
        "[end-icode-block:::False]",
        "[para(2,1):]",
        "[text(2,1):bbb:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<pre><code>aaa
</code></pre>
<p>bbb</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_blocks_196x():
    """
    Test case 196:  Final spaces are stripped before inline parsing, so a paragraph that ends with two or more spaces will not end with a hard line break.
    """

    # Arrange
    source_markdown = """aaa\a\a\a\a\a
bbb     """.replace(
        "\a", " "
    )
    expected_tokens = [
        "[para(1,1):\n:     ]",
        "[text(1,1):aaa:]",
        "[hard-break(1,4):     :\n]",
        "[text(2,1):bbb:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>aaa<br />
bbb</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_blocks_196a():
    """
    Test case 196a:  Modification of 196, but with tabs instead of space.
    """

    # Arrange
    source_markdown = """aaa\t\t\t\t\t
bbb\t\t\t\t\t"""
    expected_tokens = [
        "[para(1,1):\n:\t\t\t\t\t]",
        "[text(1,1):aaa:]",
        "[hard-break(1,4):                    :\n]",
        "[text(2,1):bbb:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>aaa<br />
bbb</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)



"""
https://github.github.com/gfm/#precedence
"""
import pytest

from .utils import act_and_assert


@pytest.mark.gfm
def test_tabs_001():
    """
    Test case 001:  (part a) a tab can be used instead of four spaces in an indented code block.
    """

    # Arrange
    source_markdown = """\tfoo\tbaz\t\tbim"""
    expected_tokens = [
        "[icode-block(1,2):\t:]",
        "[text(1,2):foo\tbaz\t\tbim:]",
        "[end-icode-block:::True]",
    ]
    expected_gfm = """<pre><code>foo\tbaz\t\tbim
</code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_tabs_002():
    """
    Test case 002:  (part b) a tab can be used instead of four spaces in an indented code block.
    """

    # Arrange
    source_markdown = """  \tfoo\tbaz\t\tbim"""
    expected_tokens = [
        "[icode-block(1,4):  \t:]",
        "[text(1,4):foo\tbaz\t\tbim:]",
        "[end-icode-block:::True]",
    ]
    expected_gfm = """<pre><code>foo\tbaz\t\tbim
</code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_tabs_002a():
    """
    Test case 002a:  002 with spaces instead of tabs
    """

    # Arrange
    source_markdown = """      foo    baz        bim"""
    expected_tokens = [
        "[icode-block(1,5):    :]",
        "[text(1,5):foo    baz        bim:  ]",
        "[end-icode-block:::True]",
    ]
    expected_gfm = """<pre><code>  foo    baz        bim
</code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_tabs_002b():
    """
    Test case 002b:  Variation of 002 tested against Babelmark
    """

    # Arrange
    source_markdown = """    a simple
      indented code block
---
      a simple
      indented code block"""
    expected_tokens = [
        "[icode-block(1,5):    :\n    ]",
        "[text(1,5):a simple\n  indented code block:]",
        "[end-icode-block:::False]",
        "[tbreak(3,1):-::---]",
        "[icode-block(4,5):    :\n    ]",
        "[text(4,5):a simple\n  indented code block:  ]",
        "[end-icode-block:::True]",
    ]
    expected_gfm = """<pre><code>a simple
  indented code block
</code></pre>
<hr />
<pre><code>  a simple
  indented code block
</code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_tabs_003():
    """
    Test case 003:  (part c) a tab can be used instead of four spaces in an indented code block.
    """

    # Arrange
    source_markdown = """    a\ta
    ὐ\ta"""
    expected_tokens = [
        "[icode-block(1,5):    :\n    ]",
        "[text(1,5):a\ta\nὐ\ta:]",
        "[end-icode-block:::True]",
    ]
    expected_gfm = """<pre><code>a\ta
ὐ\ta
</code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_tabs_004x():
    """
    Test case 004:  (part a) a continuation paragraph of a list item is indented with a tab; this has exactly the same effect as indentation with four spaces would
    """

    # Arrange
    source_markdown = """  - foo

\tbar"""  # noqa: E101,W191
    # noqa: E101,W191
    expected_tokens = [
        "[ulist(1,3):-::4:  :\t]",
        "[para(1,5):]",
        "[text(1,5):foo:]",
        "[end-para:::True]",
        "[BLANK(2,1):]",
        "[para(3,2):]",
        "[text(3,2):bar:]",
        "[end-para:::True]",
        "[end-ulist:::True]",
    ]
    expected_gfm = """<ul>
<li>
<p>foo</p>
<p>bar</p>
</li>
</ul>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_tabs_004a():
    """
    Test case 004a:  variation on 004
    """

    # Arrange
    source_markdown = """  - foo

    bar"""  # noqa: E101,W191
    # noqa: E101,W191
    expected_tokens = [
        "[ulist(1,3):-::4:  :    ]",
        "[para(1,5):]",
        "[text(1,5):foo:]",
        "[end-para:::True]",
        "[BLANK(2,1):]",
        "[para(3,5):]",
        "[text(3,5):bar:]",
        "[end-para:::True]",
        "[end-ulist:::True]",
    ]
    expected_gfm = """<ul>
<li>
<p>foo</p>
<p>bar</p>
</li>
</ul>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_tabs_005():
    """
    Test case 005:  (part b) a continuation paragraph of a list item is indented with a tab; this has exactly the same effect as indentation with four spaces would
    """

    # Arrange
    source_markdown = """- foo

\t\tbar"""  # noqa: E101,W191
    # noqa: E101,W191
    expected_tokens = [
        "[ulist(1,1):-::2::\t]",
        "[para(1,3):]",
        "[text(1,3):foo:]",
        "[end-para:::True]",
        "[BLANK(2,1):]",
        "[icode-block(3,2):\t\t:]",
        "[text(3,2):bar:  ]",
        "[end-icode-block:::True]",
        "[end-ulist:::True]",
    ]
    expected_gfm = """<ul>
<li>
<p>foo</p>
<pre><code>  bar
</code></pre>
</li>
</ul>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_tabs_006():
    """
    Test case 006:  case > is followed by a tab, which is treated as if it were expanded into three spaces.
    """

    # Arrange
    source_markdown = """>\t\tfoo"""
    expected_tokens = [
        "[block-quote(1,1)::> ]",
        "[icode-block(1,3):\t\t:]",
        "[text(1,3):foo:  ]",
        "[end-icode-block:::True]",
        "[end-block-quote:::True]",
    ]
    expected_gfm = """<blockquote>
<pre><code>  foo
</code></pre>
</blockquote>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_tabs_006a():
    """
    Test case 006a:  modified version of 006 with spaces leading in instead of tab
    """

    # Arrange
    source_markdown = """>   \tfoo"""
    expected_tokens = [
        "[block-quote(1,1)::> ]",
        "[icode-block(1,5):  \t:]",
        "[text(1,5):foo:  ]",
        "[end-icode-block:::True]",
        "[end-block-quote:::True]",
    ]
    expected_gfm = """<blockquote>
<pre><code>  foo
</code></pre>
</blockquote>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_tabs_006b():
    """
    Test case 006b:  modified version of 006 with spaces leading in instead of tab
    """

    # Arrange
    source_markdown = """>\t    foo"""
    expected_tokens = [
        "[block-quote(1,1)::> ]",
        "[icode-block(1,5):\t  :]",
        "[text(1,5):foo:  ]",
        "[end-icode-block:::True]",
        "[end-block-quote:::True]",
    ]
    expected_gfm = """<blockquote>
<pre><code>  foo
</code></pre>
</blockquote>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_tabs_006c():
    """
    Test case 006c:  modified version of 006 with spaces leading in instead of tab
    """

    # Arrange
    source_markdown = """>       foo"""
    expected_tokens = [
        "[block-quote(1,1)::> ]",
        "[icode-block(1,7):    :]",
        "[text(1,7):foo:  ]",
        "[end-icode-block:::True]",
        "[end-block-quote:::True]",
    ]
    expected_gfm = """<blockquote>
<pre><code>  foo
</code></pre>
</blockquote>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_tabs_007():
    """
    Test case 007:  none
    """

    # Arrange
    source_markdown = """-\t\tfoo"""
    expected_tokens = [
        "[ulist(1,1):-::2:]",
        "[icode-block(1,3):\t\t:]",
        "[text(1,3):foo:  ]",
        "[end-icode-block:::True]",
        "[end-ulist:::True]",
    ]
    expected_gfm = """<ul>
<li>
<pre><code>  foo
</code></pre>
</li>
</ul>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_tabs_007a():
    """
    Test case 007a:  variation on 007 with ordered list
    """

    # Arrange
    source_markdown = """1)\t\tfoo"""
    expected_tokens = [
        "[olist(1,1):):1:3:]",
        "[icode-block(1,4):\t\t:]",
        "[text(1,4):foo: ]",
        "[end-icode-block:::True]",
        "[end-olist:::True]",
    ]
    expected_gfm = """<ol>
<li>
<pre><code> foo
</code></pre>
</li>
</ol>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_tabs_007b():
    """
    Test case 007b:  variation on 007 with ordered list
    """

    # Arrange
    source_markdown = """01)\t\tfoo"""
    expected_tokens = [
        "[olist(1,1):):01:4:]",
        "[icode-block(1,5):\t\t:]",
        "[text(1,5):foo:]",
        "[end-icode-block:::True]",
        "[end-olist:::True]",
    ]
    expected_gfm = """<ol start="1">
<li>
<pre><code>foo
</code></pre>
</li>
</ol>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_tabs_007c():
    """
    Test case 007c:  variation on 007 with ordered list
    """

    # Arrange
    source_markdown = """001)\t\tfoo"""
    expected_tokens = [
        "[olist(1,1):):001:5:]",
        "[icode-block(1,6):\t\t:]",
        "[text(1,6):foo:   ]",
        "[end-icode-block:::True]",
        "[end-olist:::True]",
    ]
    expected_gfm = """<ol start="1">
<li>
<pre><code>   foo
</code></pre>
</li>
</ol>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_tabs_008():
    """
    Test case 008:  none
    """

    # Arrange
    source_markdown = """    foo
\tbar"""  # noqa: E101,W191
    # noqa: E101
    expected_tokens = [
        "[icode-block(1,5):    :\n\t]",
        "[text(1,5):foo\nbar:]",
        "[end-icode-block:::True]",
    ]
    expected_gfm = """<pre><code>foo
bar
</code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_tabs_009():
    """
    Test case 009:  none
    """

    # Arrange
    source_markdown = """ - foo
   - bar
\t - baz"""  # noqa: E101,W191
    # noqa: E101
    expected_tokens = [
        "[ulist(1,2):-::3: ]",
        "[para(1,4):]",
        "[text(1,4):foo:]",
        "[end-para:::True]",
        "[ulist(2,4):-::5:   ]",
        "[para(2,6):]",
        "[text(2,6):bar:]",
        "[end-para:::True]",
        "[ulist(3,3):-::7:\t ]",
        "[para(3,5):]",
        "[text(3,5):baz:]",
        "[end-para:::True]",
        "[end-ulist:::True]",
        "[end-ulist:::True]",
        "[end-ulist:::True]",
    ]
    expected_gfm = """<ul>
<li>foo
<ul>
<li>bar
<ul>
<li>baz</li>
</ul>
</li>
</ul>
</li>
</ul>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_tabs_010():
    """
    Test case 010:  none
    """

    # Arrange
    source_markdown = """#\tFoo"""
    expected_tokens = ["[para(1,1):]", "[text(1,1):#    Foo:]", "[end-para:::True]"]
    expected_gfm = """<p>#    Foo</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens, show_debug=False)


@pytest.mark.gfm
def test_tabs_011():
    """
    Test case 011:  none
    """

    # Arrange
    source_markdown = """*\t*\t*\t"""
    expected_tokens = ["[tbreak(1,1):*::*    *    *    ]"]
    expected_gfm = """<hr />"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)



"""
https://github.github.com/gfm/#hard-line-breaks
"""

import pytest

from .utils import act_and_assert


# pylint: disable=too-many-lines
@pytest.mark.gfm
def test_hard_line_breaks_654():
    """
    Test case 654:  A line break (not in a code span or HTML tag) that is preceded by two or more spaces and does not occur at the end of a block is parsed as a hard line break (rendered in HTML as a <br /> tag):
    """

    # Arrange
    source_markdown = """foo\a\a
baz""".replace(
        "\a", " "
    )
    expected_tokens = [
        "[para(1,1):\n]",
        "[text(1,1):foo:]",
        "[hard-break(1,4):  :\n]",
        "[text(2,1):baz:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>foo<br />
baz</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens, show_debug=False)


@pytest.mark.gfm
def test_hard_line_breaks_655x():
    """
    Test case 655:  For a more visible alternative, a backslash before the line ending may be used instead of two spaces:
    """

    # Arrange
    source_markdown = """foo\\
baz"""
    expected_tokens = [
        "[para(1,1):\n]",
        "[text(1,1):foo:]",
        "[hard-break(1,4):\\:\n]",
        "[text(2,1):baz:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>foo<br />
baz</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_655a():
    """
    Test case 655a:  variation with two backslashes at end
    """

    # Arrange
    source_markdown = """foo\\\\
baz"""
    expected_tokens = [
        "[para(1,1):\n]",
        "[text(1,1):foo\\\b\\\nbaz::\n]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>foo\\
baz</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_656():
    """
    Test case 656:  More than two spaces can be used:
    """

    # Arrange
    source_markdown = """foo\a\a\a\a\a\a\a
baz""".replace(
        "\a", " "
    )
    expected_tokens = [
        "[para(1,1):\n]",
        "[text(1,1):foo:]",
        "[hard-break(1,4):       :\n]",
        "[text(2,1):baz:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>foo<br />
baz</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_657():
    """
    Test case 657:  (part 1) Leading spaces at the beginning of the next line are ignored:
    """

    # Arrange
    source_markdown = """foo\a\a
     bar""".replace(
        "\a", " "
    )
    expected_tokens = [
        "[para(1,1):\n     ]",
        "[text(1,1):foo:]",
        "[hard-break(1,4):  :\n]",
        "[text(2,6):bar:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>foo<br />
bar</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_658():
    """
    Test case 658:  (part 2) Leading spaces at the beginning of the next line are ignored:
    """

    # Arrange
    source_markdown = """foo\\
     bar"""
    expected_tokens = [
        "[para(1,1):\n     ]",
        "[text(1,1):foo:]",
        "[hard-break(1,4):\\:\n]",
        "[text(2,6):bar:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>foo<br />
bar</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_659():
    """
    Test case 659:  (part 1) Line breaks can occur inside emphasis, links, and other constructs that allow inline content:
    """

    # Arrange
    source_markdown = """*foo\a\a
bar*""".replace(
        "\a", " "
    )
    expected_tokens = [
        "[para(1,1):\n]",
        "[emphasis(1,1):1:*]",
        "[text(1,2):foo:]",
        "[hard-break(1,5):  :\n]",
        "[text(2,1):bar:]",
        "[end-emphasis(2,4)::]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p><em>foo<br />
bar</em></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_660():
    """
    Test case 660:  (part 2) Line breaks can occur inside emphasis, links, and other constructs that allow inline content:
    """

    # Arrange
    source_markdown = """*foo\\
bar*""".replace(
        "\a", " "
    )
    expected_tokens = [
        "[para(1,1):\n]",
        "[emphasis(1,1):1:*]",
        "[text(1,2):foo:]",
        "[hard-break(1,5):\\:\n]",
        "[text(2,1):bar:]",
        "[end-emphasis(2,4)::]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p><em>foo<br />
bar</em></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_661():
    """
    Test case 661:  (part 1) Line breaks do not occur inside code spans
    """

    # Arrange
    source_markdown = """`code\a\a
span`""".replace(
        "\a", " "
    )
    expected_tokens = [
        "[para(1,1):\n]",
        "[icode-span(1,1):code  \a\n\a \aspan:`::]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p><code>code   span</code></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_662():
    """
    Test case 662:  (part 2) Line breaks do not occur inside code spans
    """

    # Arrange
    source_markdown = """`code\\
span`"""
    expected_tokens = [
        "[para(1,1):\n]",
        "[icode-span(1,1):code\\\a\n\a \aspan:`::]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p><code>code\\ span</code></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_663():
    """
    Test case 663:  (part 1) or HTML tags:
    """

    # Arrange
    source_markdown = """<a href="foo\a\a
bar">""".replace(
        "\a", " "
    )
    expected_tokens = [
        "[para(1,1):\n]",
        '[raw-html(1,1):a href="foo  \nbar"]',
        "[end-para:::True]",
    ]
    expected_gfm = """<p><a href="foo\a\a
bar"></p>""".replace(
        "\a", " "
    )

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_664():
    """
    Test case 664:  (part 2) or HTML tags:
    """

    # Arrange
    source_markdown = """<a href="foo\\
bar">"""
    expected_tokens = [
        "[para(1,1):\n]",
        '[raw-html(1,1):a href="foo\\\nbar"]',
        "[end-para:::True]",
    ]
    expected_gfm = """<p><a href="foo\\
bar"></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_665():
    """
    Test case 665:  (part 1) Neither syntax for hard line breaks works at the end of a paragraph or other block element:
    """

    # Arrange
    source_markdown = """foo\\"""
    expected_tokens = ["[para(1,1):]", "[text(1,1):foo\\:]", "[end-para:::True]"]
    expected_gfm = """<p>foo\\</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_665a():
    """
    Test case 665a:  variation of 665 with a space before hb character
    """

    # Arrange
    source_markdown = """foo \\"""
    expected_tokens = ["[para(1,1):]", "[text(1,1):foo \\:]", "[end-para:::True]"]
    expected_gfm = """<p>foo \\</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_666():
    """
    Test case 666:  (part 2) Neither syntax for hard line breaks works at the end of a paragraph or other block element:
    """

    # Arrange
    source_markdown = """foo  """
    expected_tokens = ["[para(1,1)::  ]", "[text(1,1):foo:]", "[end-para:::True]"]
    expected_gfm = """<p>foo</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_667():
    """
    Test case 667:  (part 3) Neither syntax for hard line breaks works at the end of a paragraph or other block element:
    """

    # Arrange
    source_markdown = """### foo\\"""
    expected_tokens = ["[atx(1,1):3:0:]", "[text(1,5):foo\\: ]", "[end-atx::]"]
    expected_gfm = """<h3>foo\\</h3>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_668():
    """
    Test case 668:  (part 4) Neither syntax for hard line breaks works at the end of a paragraph or other block element:
    """

    # Arrange
    source_markdown = """### foo  """
    expected_tokens = ["[atx(1,1):3:0:]", "[text(1,5):foo: ]", "[end-atx:  :]"]
    expected_gfm = """<h3>foo</h3>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_extra_01x():
    """
    Test case extra 01x:  Hard line breaks at start of line
    """

    # Arrange
    source_markdown = """\a\a
This is new.""".replace(
        "\a", " "
    )
    expected_tokens = [
        "[BLANK(1,1):  ]",
        "[para(2,1):]",
        "[text(2,1):This is new.:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>This is new.</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_extra_01a():
    """
    Test case extra 01x:  Hard line breaks at start of line
    """

    # Arrange
    source_markdown = """\\
This is new."""
    expected_tokens = [
        "[para(1,1):\n]",
        "[hard-break(1,1):\\:\n]",
        "[text(2,1):This is new.:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p><br />
This is new.</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_extra_02x():
    """
    Test case extra 01x:  Hard line breaks at start of line
    """

    # Arrange
    source_markdown = """abc
\a\a
This is new.""".replace(
        "\a", " "
    )
    expected_tokens = [
        "[para(1,1):]",
        "[text(1,1):abc:]",
        "[end-para:::True]",
        "[BLANK(2,1):  ]",
        "[para(3,1):]",
        "[text(3,1):This is new.:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>abc</p>
<p>This is new.</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_extra_02a():
    """
    Test case extra 01x:  Hard line breaks at start of line
    """

    # Arrange
    source_markdown = """abc
\\
This is new."""
    expected_tokens = [
        "[para(1,1):\n\n]",
        "[text(1,1):abc\n::\n]",
        "[hard-break(2,1):\\:\n]",
        "[text(3,1):This is new.:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>abc
<br />
This is new.</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_extra_03x():
    """
    Test case extra 01x:  Hard line break followed by text
    """

    # Arrange
    source_markdown = """abc\\
This is new."""
    expected_tokens = [
        "[para(1,1):\n]",
        "[text(1,1):abc:]",
        "[hard-break(1,4):\\:\n]",
        "[text(2,1):This is new.:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>abc<br />
This is new.</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_extra_03a():
    """
    Test case extra 01x:  Hard line break followed by backslash escaped
    """

    # Arrange
    source_markdown = """abc\\
\\\\This is new."""
    expected_tokens = [
        "[para(1,1):\n]",
        "[text(1,1):abc:]",
        "[hard-break(1,4):\\:\n]",
        "[text(2,1):\\\b\\This is new.:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>abc<br />
\\This is new.</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_extra_03b():
    """
    Test case extra 01x:  Hard line break followed by entity
    """

    # Arrange
    source_markdown = """abc\\
&copy; This is new."""
    expected_tokens = [
        "[para(1,1):\n]",
        "[text(1,1):abc:]",
        "[hard-break(1,4):\\:\n]",
        "[text(2,1):\a&copy;\a©\a This is new.:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>abc<br />
© This is new.</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens, show_debug=False)


@pytest.mark.gfm
def test_hard_line_breaks_extra_03c():
    """
    Test case extra 01x:  Hard line break followed by code span
    """

    # Arrange
    source_markdown = """abc\\
`This` is new."""
    expected_tokens = [
        "[para(1,1):\n]",
        "[text(1,1):abc:]",
        "[hard-break(1,4):\\:\n]",
        "[icode-span(2,1):This:`::]",
        "[text(2,7): is new.:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>abc<br />
<code>This</code> is new.</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_extra_03d():
    """
    Test case extra 01x:  Hard line break followed by emphasis
    """

    # Arrange
    source_markdown = """abc\\
*This* is new."""
    expected_tokens = [
        "[para(1,1):\n]",
        "[text(1,1):abc:]",
        "[hard-break(1,4):\\:\n]",
        "[emphasis(2,1):1:*]",
        "[text(2,2):This:]",
        "[end-emphasis(2,6)::]",
        "[text(2,7): is new.:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>abc<br />
<em>This</em> is new.</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_extra_03e():
    """
    Test case extra 01x:  Hard line break followed by link
    """

    # Arrange
    source_markdown = """abc\\
[This](foo.com) is new."""
    expected_tokens = [
        "[para(1,1):\n]",
        "[text(1,1):abc:]",
        "[hard-break(1,4):\\:\n]",
        "[link(2,1):inline:foo.com:::::This:False::::]",
        "[text(2,2):This:]",
        "[end-link::]",
        "[text(2,16): is new.:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>abc<br />
<a href="foo.com">This</a> is new.</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_extra_03f():
    """
    Test case extra 01x:  Hard line break followed by link
    """

    # Arrange
    source_markdown = """abc\\
![This](foo.com) is new."""
    expected_tokens = [
        "[para(1,1):\n]",
        "[text(1,1):abc:]",
        "[hard-break(1,4):\\:\n]",
        "[image(2,1):inline:foo.com::This::::This:False::::]",
        "[text(2,17): is new.:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>abc<br />
<img src="foo.com" alt="This" /> is new.</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_extra_03g():
    """
    Test case extra 01x:  Hard line break followed by autolink
    """

    # Arrange
    source_markdown = """abc\\
<http://this.com> is new."""
    expected_tokens = [
        "[para(1,1):\n]",
        "[text(1,1):abc:]",
        "[hard-break(1,4):\\:\n]",
        "[uri-autolink(2,1):http://this.com]",
        "[text(2,18): is new.:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>abc<br />
<a href="http://this.com">http://this.com</a> is new.</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_extra_03h():
    """
    Test case extra 01x:  Hard line break followed by autolink
    """

    # Arrange
    source_markdown = """abc\\
<this> is new."""
    expected_tokens = [
        "[para(1,1):\n]",
        "[text(1,1):abc:]",
        "[hard-break(1,4):\\:\n]",
        "[raw-html(2,1):this]",
        "[text(2,7): is new.:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>abc<br />
<this> is new.</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_extra_03i():
    """
    Test case extra 01x:  Hard line break followed by hard line break
    """

    # Arrange
    source_markdown = """abc\\
\\
this is new."""
    expected_tokens = [
        "[para(1,1):\n\n]",
        "[text(1,1):abc:]",
        "[hard-break(1,4):\\:\n]",
        "[hard-break(2,1):\\:\n]",
        "[text(3,1):this is new.:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>abc<br />
<br />
this is new.</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_extra_04x():
    """
    Test case extra 01x:  Hard line break followed by text
    """

    # Arrange
    source_markdown = """abc\a\a
This is new.""".replace(
        "\a", " "
    )
    expected_tokens = [
        "[para(1,1):\n]",
        "[text(1,1):abc:]",
        "[hard-break(1,4):  :\n]",
        "[text(2,1):This is new.:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>abc<br />
This is new.</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_extra_04xa():
    """
    Test case extra 01x:  Hard line break followed by text
    """

    # Arrange
    source_markdown = """abc\a\a
  This is new.""".replace(
        "\a", " "
    )
    expected_tokens = [
        "[para(1,1):\n  ]",
        "[text(1,1):abc:]",
        "[hard-break(1,4):  :\n]",
        "[text(2,3):This is new.:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>abc<br />
This is new.</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_extra_04a():
    """
    Test case extra 01x:  Hard line break followed by backslash escaped
    """

    # Arrange
    source_markdown = """abc\a\a
\\\\This is new.""".replace(
        "\a", " "
    )
    expected_tokens = [
        "[para(1,1):\n]",
        "[text(1,1):abc:]",
        "[hard-break(1,4):  :\n]",
        "[text(2,1):\\\b\\This is new.:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>abc<br />
\\This is new.</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_extra_04b():
    """
    Test case extra 01x:  Hard line break followed by entity
    """

    # Arrange
    source_markdown = """abc\a\a
&copy; This is new.""".replace(
        "\a", " "
    )
    expected_tokens = [
        "[para(1,1):\n]",
        "[text(1,1):abc:]",
        "[hard-break(1,4):  :\n]",
        "[text(2,1):\a&copy;\a©\a This is new.:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>abc<br />
© This is new.</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_extra_04c():
    """
    Test case extra 01x:  Hard line break followed by code span
    """

    # Arrange
    source_markdown = """abc\a\a
`This` is new.""".replace(
        "\a", " "
    )
    expected_tokens = [
        "[para(1,1):\n]",
        "[text(1,1):abc:]",
        "[hard-break(1,4):  :\n]",
        "[icode-span(2,1):This:`::]",
        "[text(2,7): is new.:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>abc<br />
<code>This</code> is new.</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_extra_04d():
    """
    Test case extra 01x:  Hard line break followed by emphasis
    """

    # Arrange
    source_markdown = """abc\a\a
*This* is new.""".replace(
        "\a", " "
    )
    expected_tokens = [
        "[para(1,1):\n]",
        "[text(1,1):abc:]",
        "[hard-break(1,4):  :\n]",
        "[emphasis(2,1):1:*]",
        "[text(2,2):This:]",
        "[end-emphasis(2,6)::]",
        "[text(2,7): is new.:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>abc<br />
<em>This</em> is new.</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_extra_04e():
    """
    Test case extra 01x:  Hard line break followed by link
    """

    # Arrange
    source_markdown = """abc\a\a
[This](foo.com) is new.""".replace(
        "\a", " "
    )
    expected_tokens = [
        "[para(1,1):\n]",
        "[text(1,1):abc:]",
        "[hard-break(1,4):  :\n]",
        "[link(2,1):inline:foo.com:::::This:False::::]",
        "[text(2,2):This:]",
        "[end-link::]",
        "[text(2,16): is new.:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>abc<br />
<a href="foo.com">This</a> is new.</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_extra_04f():
    """
    Test case extra 01x:  Hard line break followed by link
    """

    # Arrange
    source_markdown = """abc\a\a
![This](foo.com) is new.""".replace(
        "\a", " "
    )
    expected_tokens = [
        "[para(1,1):\n]",
        "[text(1,1):abc:]",
        "[hard-break(1,4):  :\n]",
        "[image(2,1):inline:foo.com::This::::This:False::::]",
        "[text(2,17): is new.:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>abc<br />
<img src="foo.com" alt="This" /> is new.</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_extra_04g():
    """
    Test case extra 01x:  Hard line break followed by autolink
    """

    # Arrange
    source_markdown = """abc\a\a
<http://this.com> is new.""".replace(
        "\a", " "
    )
    expected_tokens = [
        "[para(1,1):\n]",
        "[text(1,1):abc:]",
        "[hard-break(1,4):  :\n]",
        "[uri-autolink(2,1):http://this.com]",
        "[text(2,18): is new.:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>abc<br />
<a href="http://this.com">http://this.com</a> is new.</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_extra_04h():
    """
    Test case extra 01x:  Hard line break followed by autolink
    """

    # Arrange
    source_markdown = """abc\a\a
<this> is new.""".replace(
        "\a", " "
    )
    expected_tokens = [
        "[para(1,1):\n]",
        "[text(1,1):abc:]",
        "[hard-break(1,4):  :\n]",
        "[raw-html(2,1):this]",
        "[text(2,7): is new.:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>abc<br />
<this> is new.</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_extra_04i():
    """
    Test case extra 01x:  Hard line break followed by hard line break
    """

    # Arrange
    source_markdown = """abc\a\a
\a\a
this is new.""".replace(
        "\a", " "
    )
    expected_tokens = [
        "[para(1,1)::  ]",
        "[text(1,1):abc:]",
        "[end-para:::True]",
        "[BLANK(2,1):  ]",
        "[para(3,1):]",
        "[text(3,1):this is new.:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>abc</p>
<p>this is new.</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_extra_05x():
    """
    Test case extra 01x:  Hard line break followed by hard line break
    """

    # Arrange
    source_markdown = """abc\a\a
   *this* is new.""".replace(
        "\a", " "
    )
    expected_tokens = [
        "[para(1,1):\n   ]",
        "[text(1,1):abc:]",
        "[hard-break(1,4):  :\n]",
        "[emphasis(2,4):1:*]",
        "[text(2,5):this:]",
        "[end-emphasis(2,9)::]",
        "[text(2,10): is new.:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>abc<br />
<em>this</em> is new.</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_hard_line_breaks_extra_05a():
    """
    Test case extra 01x:  Hard line break followed by hard line break
    """

    # Arrange
    source_markdown = """abc\\
   *this* is new."""
    expected_tokens = [
        "[para(1,1):\n   ]",
        "[text(1,1):abc:]",
        "[hard-break(1,4):\\:\n]",
        "[emphasis(2,4):1:*]",
        "[text(2,5):this:]",
        "[end-emphasis(2,9)::]",
        "[text(2,10): is new.:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>abc<br />
<em>this</em> is new.</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)



"""
https://github.github.com/gfm/#emphasis-and-strong-emphasis
"""
import pytest

from .utils import act_and_assert


@pytest.mark.gfm
def test_emphasis_431():
    """
    Test case 431:  (part 1) Any nonempty sequence of inline elements can be the contents of an strongly emphasized span.
    """

    # Arrange
    source_markdown = """**foo [bar](/url)**"""
    expected_tokens = [
        "[para(1,1):]",
        "[emphasis(1,1):2:*]",
        "[text(1,3):foo :]",
        "[link(1,7):inline:/url:::::bar:False::::]",
        "[text(1,8):bar:]",
        "[end-link::]",
        "[end-emphasis(1,18)::]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p><strong>foo <a href="/url">bar</a></strong></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_emphasis_432():
    """
    Test case 432:  (part 2) Any nonempty sequence of inline elements can be the contents of an strongly emphasized span.
    """

    # Arrange
    source_markdown = """**foo
bar**"""
    expected_tokens = [
        "[para(1,1):\n]",
        "[emphasis(1,1):2:*]",
        "[text(1,3):foo\nbar::\n]",
        "[end-emphasis(2,4)::]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p><strong>foo
bar</strong></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_emphasis_433():
    """
    Test case 433:  (part 1) In particular, emphasis and strong emphasis can be nested inside strong emphasis:
    """

    # Arrange
    source_markdown = """__foo _bar_ baz__"""
    expected_tokens = [
        "[para(1,1):]",
        "[emphasis(1,1):2:_]",
        "[text(1,3):foo :]",
        "[emphasis(1,7):1:_]",
        "[text(1,8):bar:]",
        "[end-emphasis(1,11)::]",
        "[text(1,12): baz:]",
        "[end-emphasis(1,16)::]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p><strong>foo <em>bar</em> baz</strong></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_emphasis_434():
    """
    Test case 434:  (part 2) In particular, emphasis and strong emphasis can be nested inside strong emphasis:
    """

    # Arrange
    source_markdown = """__foo __bar__ baz__"""
    expected_tokens = [
        "[para(1,1):]",
        "[emphasis(1,1):2:_]",
        "[text(1,3):foo :]",
        "[emphasis(1,7):2:_]",
        "[text(1,9):bar:]",
        "[end-emphasis(1,12)::]",
        "[text(1,14): baz:]",
        "[end-emphasis(1,18)::]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p><strong>foo <strong>bar</strong> baz</strong></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_emphasis_435():
    """
    Test case 435:  (part 3) In particular, emphasis and strong emphasis can be nested inside strong emphasis:
    """

    # Arrange
    source_markdown = """____foo__ bar__"""
    expected_tokens = [
        "[para(1,1):]",
        "[emphasis(1,1):2:_]",
        "[emphasis(1,3):2:_]",
        "[text(1,5):foo:]",
        "[end-emphasis(1,8)::]",
        "[text(1,10): bar:]",
        "[end-emphasis(1,14)::]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p><strong><strong>foo</strong> bar</strong></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_emphasis_436():
    """
    Test case 436:  (part 4) In particular, emphasis and strong emphasis can be nested inside strong emphasis:
    """

    # Arrange
    source_markdown = """**foo **bar****"""
    expected_tokens = [
        "[para(1,1):]",
        "[emphasis(1,1):2:*]",
        "[text(1,3):foo :]",
        "[emphasis(1,7):2:*]",
        "[text(1,9):bar:]",
        "[end-emphasis(1,12)::]",
        "[end-emphasis(1,14)::]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p><strong>foo <strong>bar</strong></strong></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_emphasis_437():
    """
    Test case 437:  (part 5) In particular, emphasis and strong emphasis can be nested inside strong emphasis:
    """

    # Arrange
    source_markdown = """**foo *bar* baz**"""
    expected_tokens = [
        "[para(1,1):]",
        "[emphasis(1,1):2:*]",
        "[text(1,3):foo :]",
        "[emphasis(1,7):1:*]",
        "[text(1,8):bar:]",
        "[end-emphasis(1,11)::]",
        "[text(1,12): baz:]",
        "[end-emphasis(1,16)::]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p><strong>foo <em>bar</em> baz</strong></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_emphasis_438():
    """
    Test case 438:  (part 6) In particular, emphasis and strong emphasis can be nested inside strong emphasis:
    """

    # Arrange
    source_markdown = """**foo*bar*baz**"""
    expected_tokens = [
        "[para(1,1):]",
        "[emphasis(1,1):2:*]",
        "[text(1,3):foo:]",
        "[emphasis(1,6):1:*]",
        "[text(1,7):bar:]",
        "[end-emphasis(1,10)::]",
        "[text(1,11):baz:]",
        "[end-emphasis(1,14)::]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p><strong>foo<em>bar</em>baz</strong></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_emphasis_439():
    """
    Test case 439:  (part 7) In particular, emphasis and strong emphasis can be nested inside strong emphasis:
    """

    # Arrange
    source_markdown = """***foo* bar**"""
    expected_tokens = [
        "[para(1,1):]",
        "[emphasis(1,1):2:*]",
        "[emphasis(1,3):1:*]",
        "[text(1,4):foo:]",
        "[end-emphasis(1,7)::]",
        "[text(1,8): bar:]",
        "[end-emphasis(1,12)::]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p><strong><em>foo</em> bar</strong></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_emphasis_440():
    """
    Test case 440:  (part 8) In particular, emphasis and strong emphasis can be nested inside strong emphasis:
    """

    # Arrange
    source_markdown = """**foo *bar***"""
    expected_tokens = [
        "[para(1,1):]",
        "[emphasis(1,1):2:*]",
        "[text(1,3):foo :]",
        "[emphasis(1,7):1:*]",
        "[text(1,8):bar:]",
        "[end-emphasis(1,11)::]",
        "[end-emphasis(1,12)::]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p><strong>foo <em>bar</em></strong></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_emphasis_441():
    """
    Test case 441:  (part 1) Indefinite levels of nesting are possible:
    """

    # Arrange
    source_markdown = """**foo *bar **baz**
bim* bop**"""
    expected_tokens = [
        "[para(1,1):\n]",
        "[emphasis(1,1):2:*]",
        "[text(1,3):foo :]",
        "[emphasis(1,7):1:*]",
        "[text(1,8):bar :]",
        "[emphasis(1,12):2:*]",
        "[text(1,14):baz:]",
        "[end-emphasis(1,17)::]",
        "[text(1,19):\nbim::\n]",
        "[end-emphasis(2,4)::]",
        "[text(2,5): bop:]",
        "[end-emphasis(2,9)::]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p><strong>foo <em>bar <strong>baz</strong>
bim</em> bop</strong></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_emphasis_442():
    """
    Test case 442:  (part 2) Indefinite levels of nesting are possible:
    """

    # Arrange
    source_markdown = """**foo [*bar*](/url)**"""
    expected_tokens = [
        "[para(1,1):]",
        "[emphasis(1,1):2:*]",
        "[text(1,3):foo :]",
        "[link(1,7):inline:/url:::::*bar*:False::::]",
        "[emphasis(1,8):1:*]",
        "[text(1,9):bar:]",
        "[end-emphasis(1,12)::]",
        "[end-link::]",
        "[end-emphasis(1,20)::]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p><strong>foo <a href="/url"><em>bar</em></a></strong></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_emphasis_443():
    """
    Test case 443:  (part 1) There can be no empty emphasis or strong emphasis:
    """

    # Arrange
    source_markdown = """__ is not an empty emphasis"""
    expected_tokens = [
        "[para(1,1):]",
        "[text(1,1):__:]",
        "[text(1,3): is not an empty emphasis:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>__ is not an empty emphasis</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_emphasis_444():
    """
    Test case 444:  (part 2) There can be no empty emphasis or strong emphasis:
    """

    # Arrange
    source_markdown = """____ is not an empty strong emphasis"""
    expected_tokens = [
        "[para(1,1):]",
        "[text(1,1):____:]",
        "[text(1,5): is not an empty strong emphasis:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>____ is not an empty strong emphasis</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)



"""
https://github.github.com/gfm/#emphasis-and-strong-emphasis
"""
import pytest

from .utils import act_and_assert


@pytest.mark.gfm
def test_emphasis_469():
    """
    Test case 469:  (part 1) Rule 13 implies that if you want emphasis nested directly inside emphasis, you must use different delimiters:
    """

    # Arrange
    source_markdown = """**foo**"""
    expected_tokens = [
        "[para(1,1):]",
        "[emphasis(1,1):2:*]",
        "[text(1,3):foo:]",
        "[end-emphasis(1,6)::]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p><strong>foo</strong></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_emphasis_470():
    """
    Test case 470:  (part 2) Rule 13 implies that if you want emphasis nested directly inside emphasis, you must use different delimiters:
    """

    # Arrange
    source_markdown = """*_foo_*"""
    expected_tokens = [
        "[para(1,1):]",
        "[emphasis(1,1):1:*]",
        "[emphasis(1,2):1:_]",
        "[text(1,3):foo:]",
        "[end-emphasis(1,6)::]",
        "[end-emphasis(1,7)::]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p><em><em>foo</em></em></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_emphasis_471():
    """
    Test case 471:  (part 3) Rule 13 implies that if you want emphasis nested directly inside emphasis, you must use different delimiters:
    """

    # Arrange
    source_markdown = """__foo__"""
    expected_tokens = [
        "[para(1,1):]",
        "[emphasis(1,1):2:_]",
        "[text(1,3):foo:]",
        "[end-emphasis(1,6)::]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p><strong>foo</strong></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_emphasis_472():
    """
    Test case 472:  (part 4) Rule 13 implies that if you want emphasis nested directly inside emphasis, you must use different delimiters:
    """

    # Arrange
    source_markdown = """_*foo*_"""
    expected_tokens = [
        "[para(1,1):]",
        "[emphasis(1,1):1:_]",
        "[emphasis(1,2):1:*]",
        "[text(1,3):foo:]",
        "[end-emphasis(1,6)::]",
        "[end-emphasis(1,7)::]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p><em><em>foo</em></em></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_emphasis_473():
    """
    Test case 473:  (part 1) However, strong emphasis within strong emphasis is possible without switching delimiters:
    """

    # Arrange
    source_markdown = """****foo****"""
    expected_tokens = [
        "[para(1,1):]",
        "[emphasis(1,1):2:*]",
        "[emphasis(1,3):2:*]",
        "[text(1,5):foo:]",
        "[end-emphasis(1,8)::]",
        "[end-emphasis(1,10)::]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p><strong><strong>foo</strong></strong></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_emphasis_474():
    """
    Test case 474:  (part 2) However, strong emphasis within strong emphasis is possible without switching delimiters:
    """

    # Arrange
    source_markdown = """____foo____"""
    expected_tokens = [
        "[para(1,1):]",
        "[emphasis(1,1):2:_]",
        "[emphasis(1,3):2:_]",
        "[text(1,5):foo:]",
        "[end-emphasis(1,8)::]",
        "[end-emphasis(1,10)::]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p><strong><strong>foo</strong></strong></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_emphasis_475():
    """
    Test case 475:  Rule 13 can be applied to arbitrarily long sequences of delimiters:
    """

    # Arrange
    source_markdown = """******foo******"""
    expected_tokens = [
        "[para(1,1):]",
        "[emphasis(1,1):2:*]",
        "[emphasis(1,3):2:*]",
        "[emphasis(1,5):2:*]",
        "[text(1,7):foo:]",
        "[end-emphasis(1,10)::]",
        "[end-emphasis(1,12)::]",
        "[end-emphasis(1,14)::]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p><strong><strong><strong>foo</strong></strong></strong></p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)



"""
Module to provide tests related to the MD018 rule.
"""
from test.markdown_scanner import MarkdownScanner

import pytest

# pylint: disable=too-many-lines


@pytest.mark.rules
def test_md018_good_start_spacing():
    """
    Test to make sure we get the expected behavior after scanning a good file from the
    test/resources/rules/md018 directory that has good atx heading start spacing after
    the first hash.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "scan",
        "test/resources/rules/md018/good_start_spacing.md",
    ]

    expected_return_code = 0
    expected_output = ""
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )


@pytest.mark.rules
def test_md018_good_start_spacing_in_list():
    """
    Test to make sure we get the expected behavior after scanning a good file from the
    test/resources/rules/md018 directory that has good atx heading start spacing after
    the first hash.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "scan",
        "test/resources/rules/md018/good_start_spacing_in_list.md",
    ]

    expected_return_code = 0
    expected_output = ""
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )


@pytest.mark.rules
def test_md018_good_start_spacing_in_block_quotes():
    """
    Test to make sure we get the expected behavior after scanning a good file from the
    test/resources/rules/md018 directory that has good atx heading start spacing after
    the first hash.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "--disable-rules",
        "md022",
        "scan",
        "test/resources/rules/md018/good_start_spacing_in_block_quote.md",
    ]

    expected_return_code = 0
    expected_output = ""
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )


@pytest.mark.rules
def test_md018_bad_ignore_bad_atx_closed_spacing():
    """
    Test to make sure we get the expected behavior after scanning a good file from the
    test/resources/rules/md018 directory that has an atx heading with no spaces after
    initial hash, but ends with a close hash, making it a closed atx.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "--disable-rules",
        "md020",
        "scan",
        "test/resources/rules/md018/ignore_bad_atx_closed_spacing.md",
    ]

    expected_return_code = 0
    expected_output = ""
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )


@pytest.mark.rules
def test_md018_bad_missing_atx_start_spacing():
    """
    Test to make sure we get the expected behavior after scanning a good file from the
    test/resources/rules/md018 directory that has an atx heading with no spaces after
    initial hash.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "scan",
        "test/resources/rules/md018/missing_start_spacing.md",
    ]

    expected_return_code = 1
    expected_output = (
        "test/resources/rules/md018/missing_start_spacing.md:1:1: "
        + "MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
        + "test/resources/rules/md018/missing_start_spacing.md:3:1: "
        + "MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
    )
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )


@pytest.mark.rules
def test_md018_bad_missing_atx_start_spacing_in_list():
    """
    Test to make sure we get the expected behavior after scanning a good file from the
    test/resources/rules/md018 directory that has an atx heading with no spaces after
    initial hash.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "scan",
        "test/resources/rules/md018/missing_start_spacing_in_list.md",
    ]

    expected_return_code = 1
    expected_output = (
        "test/resources/rules/md018/missing_start_spacing_in_list.md:1:4: "
        + "MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
        + "test/resources/rules/md018/missing_start_spacing_in_list.md:3:4: "
        + "MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
    )
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )


@pytest.mark.rules
def test_md018_bad_missing_atx_start_spacing_in_block_quote():
    """
    Test to make sure we get the expected behavior after scanning a good file from the
    test/resources/rules/md018 directory that has an atx heading with no spaces after
    initial hash.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "scan",
        "test/resources/rules/md018/missing_start_spacing_in_block_quote.md",
    ]

    expected_return_code = 1
    expected_output = (
        "test/resources/rules/md018/missing_start_spacing_in_block_quote.md:1:3: MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
        + "test/resources/rules/md018/missing_start_spacing_in_block_quote.md:3:3: MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
    )
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )


@pytest.mark.rules
def test_md018_good_with_setext_headings():
    """
    Test to make sure we get the expected behavior after scanning a good file from the
    test/resources/rules/md018 directory that has a possible atx heading except that
    it is followed by setext headings.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "scan",
        "test/resources/rules/md018/with_setext_headings.md",
    ]

    expected_return_code = 0
    expected_output = ""
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )


@pytest.mark.rules
def test_md018_good_with_code_blocks():
    """
    Test to make sure we get the expected behavior after scanning a good file from the
    test/resources/rules/md018 directory that has a possible atx heading except that
    it is followed by code blocks.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "scan",
        "test/resources/rules/md018/with_code_blocks.md",
    ]

    expected_return_code = 0
    expected_output = ""
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )


@pytest.mark.rules
def test_md018_good_with_html_blocks():
    """
    Test to make sure we get the expected behavior after scanning a good file from the
    test/resources/rules/md018 directory that has a possible atx heading except that
    it is followed by html blocks.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "scan",
        "test/resources/rules/md018/with_html_blocks.md",
    ]

    expected_return_code = 0
    expected_output = ""
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )


@pytest.mark.rules
def test_md018_bad_multiple_within_paragraph():
    """
    Test to make sure we get the expected behavior after scanning a good file from the
    test/resources/rules/md018 directory that has multiple possible atx headings within
    a single paragraph.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "scan",
        "test/resources/rules/md018/multiple_within_paragraph.md",
    ]

    expected_return_code = 1
    expected_output = (
        "test/resources/rules/md018/multiple_within_paragraph.md:1:1: "
        + "MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
        + "test/resources/rules/md018/multiple_within_paragraph.md:2:1: "
        + "MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
    )
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )


@pytest.mark.rules
def test_md018_bad_multiple_within_paragraph_separated_codespan():
    """
    Test to make sure we get the expected behavior after scanning a good file from the
    test/resources/rules/md018 directory that has multiple possible atx headings within
    a single paragraph.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "--stack-trace",
        "--disable-rules",
        "md020",
        "scan",
        "test/resources/rules/md018/multiple_within_paragraph_separated_codespan.md",
    ]

    expected_return_code = 1
    expected_output = (
        "test/resources/rules/md018/multiple_within_paragraph_separated_codespan.md:1:1: "
        + "MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
        + "test/resources/rules/md018/multiple_within_paragraph_separated_codespan.md:3:1: "
        + "MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
    )
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )


@pytest.mark.rules
def test_md018_bad_multiple_within_paragraph_separated_codespan_multi():
    """
    Test to make sure we get the expected behavior after scanning a good file from the
    test/resources/rules/md018 directory that has multiple possible atx headings within
    a single paragraph.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "--stack-trace",
        "--disable-rules",
        "md020",
        "scan",
        "test/resources/rules/md018/multiple_within_paragraph_separated_codespan_multi.md",
    ]

    expected_return_code = 1
    expected_output = (
        "test/resources/rules/md018/multiple_within_paragraph_separated_codespan_multi.md:1:1: "
        + "MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
        + "test/resources/rules/md018/multiple_within_paragraph_separated_codespan_multi.md:4:1: "
        + "MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
    )
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )


@pytest.mark.rules
def test_md018_bad_multiple_within_paragraph_separated_inline_codespan_multi():
    """
    Test to make sure we get the expected behavior after scanning a good file from the
    test/resources/rules/md018 directory that has multiple possible atx headings within
    a single paragraph.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "--stack-trace",
        "--disable-rules",
        "md020",
        "scan",
        "test/resources/rules/md018/multiple_within_paragraph_separated_inline_codespan_multi.md",
    ]

    expected_return_code = 1
    expected_output = (
        "test/resources/rules/md018/multiple_within_paragraph_separated_inline_codespan_multi.md:4:3: "
        + "MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
    )
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )


@pytest.mark.rules
def test_md018_bad_multiple_within_paragraph_separated_inline_rawhtml_multi():
    """
    Test to make sure we get the expected behavior after scanning a good file from the
    test/resources/rules/md018 directory that has multiple possible atx headings within
    a single paragraph.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "--stack-trace",
        "--disable-rules",
        "md020",
        "scan",
        "test/resources/rules/md018/multiple_within_paragraph_separated_inline_rawhtml_multi.md",
    ]

    expected_return_code = 1
    expected_output = (
        "test/resources/rules/md018/multiple_within_paragraph_separated_inline_rawhtml_multi.md:4:3: "
        + "MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
    )
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )


@pytest.mark.rules
def test_md018_bad_multiple_within_paragraph_separated_inline_image_multi():
    """
    Test to make sure we get the expected behavior after scanning a good file from the
    test/resources/rules/md018 directory that has multiple possible atx headings within
    a single paragraph.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "--stack-trace",
        "--disable-rules",
        "md020",
        "scan",
        "test/resources/rules/md018/multiple_within_paragraph_separated_inline_image_multi.md",
    ]

    expected_return_code = 1
    expected_output = (
        "test/resources/rules/md018/multiple_within_paragraph_separated_inline_image_multi.md:8:3: "
        + "MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
    )
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )


@pytest.mark.rules
def test_md018_bad_multiple_within_paragraph_separated_full_image_multi():
    """
    Test to make sure we get the expected behavior after scanning a good file from the
    test/resources/rules/md018 directory that has multiple possible atx headings within
    a single paragraph.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "--stack-trace",
        "--disable-rules",
        "md020",
        "scan",
        "test/resources/rules/md018/multiple_within_paragraph_separated_full_image_multi.md",
    ]

    expected_return_code = 1
    expected_output = (
        "test/resources/rules/md018/multiple_within_paragraph_separated_full_image_multi.md:5:3: "
        + "MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
    )
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )


@pytest.mark.rules
def test_md018_bad_multiple_within_paragraph_separated_shortcut_image_multi():
    """
    Test to make sure we get the expected behavior after scanning a good file from the
    test/resources/rules/md018 directory that has multiple possible atx headings within
    a single paragraph.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "--stack-trace",
        "--disable-rules",
        "md020",
        "scan",
        "test/resources/rules/md018/multiple_within_paragraph_separated_shortcut_image_multi.md",
    ]

    expected_return_code = 1
    expected_output = (
        "test/resources/rules/md018/multiple_within_paragraph_separated_shortcut_image_multi.md:4:3: "
        + "MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
    )
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )


@pytest.mark.rules
def test_md018_bad_multiple_within_paragraph_separated_collapsed_image_multi():
    """
    Test to make sure we get the expected behavior after scanning a good file from the
    test/resources/rules/md018 directory that has multiple possible atx headings within
    a single paragraph.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "--stack-trace",
        "--disable-rules",
        "md020",
        "scan",
        "test/resources/rules/md018/multiple_within_paragraph_separated_collapsed_image_multi.md",
    ]

    expected_return_code = 1
    expected_output = (
        "test/resources/rules/md018/multiple_within_paragraph_separated_collapsed_image_multi.md:4:3: "
        + "MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
    )
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )


@pytest.mark.rules
def test_md018_bad_multiple_within_paragraph_separated_inline_link_multi():
    """
    Test to make sure we get the expected behavior after scanning a good file from the
    test/resources/rules/md018 directory that has multiple possible atx headings within
    a single paragraph.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "--stack-trace",
        "--disable-rules",
        "md020",
        "scan",
        "test/resources/rules/md018/multiple_within_paragraph_separated_inline_link_multi.md",
    ]

    expected_return_code = 1
    expected_output = (
        "test/resources/rules/md018/multiple_within_paragraph_separated_inline_link_multi.md:8:3: "
        + "MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
    )
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )


@pytest.mark.rules
def test_md018_bad_multiple_within_paragraph_separated_full_link_multi():
    """
    Test to make sure we get the expected behavior after scanning a good file from the
    test/resources/rules/md018 directory that has multiple possible atx headings within
    a single paragraph.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "--stack-trace",
        "--disable-rules",
        "md020",
        "scan",
        "test/resources/rules/md018/multiple_within_paragraph_separated_full_link_multi.md",
    ]

    expected_return_code = 1
    expected_output = (
        "test/resources/rules/md018/multiple_within_paragraph_separated_full_link_multi.md:5:3: "
        + "MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
    )
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )


@pytest.mark.rules
def test_md018_bad_multiple_within_paragraph_separated_shortcut_link_multi():
    """
    Test to make sure we get the expected behavior after scanning a good file from the
    test/resources/rules/md018 directory that has multiple possible atx headings within
    a single paragraph.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "--stack-trace",
        "--disable-rules",
        "md020",
        "scan",
        "test/resources/rules/md018/multiple_within_paragraph_separated_shortcut_link_multi.md",
    ]

    expected_return_code = 1
    expected_output = (
        "test/resources/rules/md018/multiple_within_paragraph_separated_shortcut_link_multi.md:4:3: "
        + "MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
    )
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )


@pytest.mark.rules
def test_md018_bad_multiple_within_paragraph_separated_collapsed_link_multi():
    """
    Test to make sure we get the expected behavior after scanning a good file from the
    test/resources/rules/md018 directory that has multiple possible atx headings within
    a single paragraph.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "--stack-trace",
        "--disable-rules",
        "md020",
        "scan",
        "test/resources/rules/md018/multiple_within_paragraph_separated_collapsed_link_multi.md",
    ]

    expected_return_code = 1
    expected_output = (
        "test/resources/rules/md018/multiple_within_paragraph_separated_collapsed_link_multi.md:4:3: "
        + "MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
    )
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )


@pytest.mark.rules
def test_md018_bad_multiple_within_paragraph_separated_inline_hardbreak_multi():
    """
    Test to make sure we get the expected behavior after scanning a good file from the
    test/resources/rules/md018 directory that has multiple possible atx headings within
    a single paragraph.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "--stack-trace",
        "--disable-rules",
        "md020",
        "scan",
        "test/resources/rules/md018/multiple_within_paragraph_separated_inline_hardbreak_multi.md",
    ]

    expected_return_code = 1
    expected_output = (
        "test/resources/rules/md018/multiple_within_paragraph_separated_inline_hardbreak_multi.md:3:3: "
        + "MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
    )
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )


@pytest.mark.rules
def test_md018_bad_paragraphs_with_starting_whitespace():
    """
    Test to make sure we get the expected behavior after scanning a good file from the
    test/resources/rules/md018 directory that has multiple possible atx headings each
    one with starting whitespace that would normally be permitted.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "scan",
        "test/resources/rules/md018/paragraphs_with_starting_whitespace.md",
    ]

    expected_return_code = 1
    expected_output = (
        "test/resources/rules/md018/paragraphs_with_starting_whitespace.md:1:1: "
        + "MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
        + "test/resources/rules/md018/paragraphs_with_starting_whitespace.md:3:2: "
        + "MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
        + "test/resources/rules/md018/paragraphs_with_starting_whitespace.md:5:3: "
        + "MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
        + "test/resources/rules/md018/paragraphs_with_starting_whitespace.md:7:4: "
        + "MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
    )
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )


@pytest.mark.rules
def test_md018_bad_single_paragraph_with_starting_space():
    """
    Test to make sure we get the expected behavior after scanning a good file from the
    test/resources/rules/md018 directory that has multiple possible atx headings within
    a single paragraph each one with starting space that would normally be permitted.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "scan",
        "test/resources/rules/md018/single_paragraph_with_starting_space.md",
    ]

    expected_return_code = 1
    expected_output = (
        "test/resources/rules/md018/single_paragraph_with_starting_space.md:1:1: "
        + "MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
        + "test/resources/rules/md018/single_paragraph_with_starting_space.md:2:2: "
        + "MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
        + "test/resources/rules/md018/single_paragraph_with_starting_space.md:3:3: "
        + "MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
        + "test/resources/rules/md018/single_paragraph_with_starting_space.md:4:4: "
        + "MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
    )
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )


@pytest.mark.rules
def test_md018_bad_single_paragraph_with_starting_whitespace():
    """
    Test to make sure we get the expected behavior after scanning a good file from the
    test/resources/rules/md018 directory that has multiple possible atx headings within
    a single paragraph each one with starting whitespace that would normally be
    permitted.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "scan",
        "test/resources/rules/md018/single_paragraph_with_starting_whitespace.md",
    ]

    expected_return_code = 1
    expected_output = (
        "test/resources/rules/md018/single_paragraph_with_starting_whitespace.md:1:1: "
        + "MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
    )
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )


@pytest.mark.rules
def test_md018_bad_single_paragraph_with_whitespace():
    """
    Test to make sure we get the expected behavior after scanning a good file from the
    test/resources/rules/md018 directory that has multiple possible atx headings within
    a single paragraph. The first one should be detected as it has a space character
    between the # and the text, but the second one should not as it contains a tab
    character between the # and the text.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "scan",
        "test/resources/rules/md018/single_paragraph_with_whitespace.md",
    ]

    expected_return_code = 1
    expected_output = (
        "test/resources/rules/md018/single_paragraph_with_whitespace.md:1:1: "
        + "MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
        + "test/resources/rules/md018/single_paragraph_with_whitespace.md:2:2: "
        + "MD018: No space present after the hash character on a possible Atx Heading. (no-missing-space-atx)\n"
    )
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )



"""
https://github.github.com/gfm/#paragraph
"""
import pytest

from .utils import act_and_assert

# pylint: disable=too-many-lines


@pytest.mark.gfm
def test_paragraph_series_m_ib_i3_ol_nl_i4_t_ib():
    """
    Test case:  Indent of 3 ordered list newline indent of 4 text (indented block)
    was:        test_list_blocks_256ix
    """

    # Arrange
    source_markdown = """   1.
    foo
"""
    expected_tokens = [
        "[olist(1,4):.:1:6:   ]",
        "[BLANK(1,6):]",
        "[end-olist:::True]",
        "[icode-block(2,5):    :]",
        "[text(2,5):foo:]",
        "[end-icode-block:::True]",
        "[BLANK(3,1):]",
    ]
    expected_gfm = """<ol>
<li></li>
</ol>
<pre><code>foo
</code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_ib_i3_ol_nl_i7_t_all_ib():
    """
    Test case:  Indent of 3 ordered list newline indent of 7 text (indented block)
    """

    # Arrange
    source_markdown = """1.
       foo
"""
    expected_tokens = [
        "[olist(1,1):.:1:3::   ]",
        "[BLANK(1,3):]",
        "[icode-block(2,8):    :]",
        "[text(2,8):foo:]",
        "[end-icode-block:::True]",
        "[BLANK(3,1):]",
        "[end-olist:::True]",
    ]
    expected_gfm = """<ol>
<li>
<pre><code>foo
</code></pre>
</li>
</ol>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_ib_i3_ol_nl_i5_t_ib():
    """
    Test case:  Indent of 3 ordered list newline indent of 5 text (indented block)
    was:        test_list_blocks_256ixa
    """

    # Arrange
    source_markdown = """   1.
     foo
"""
    expected_tokens = [
        "[olist(1,4):.:1:6:   ]",
        "[BLANK(1,6):]",
        "[end-olist:::True]",
        "[icode-block(2,5):    :]",
        "[text(2,5):foo: ]",
        "[end-icode-block:::True]",
        "[BLANK(3,1):]",
    ]
    expected_gfm = """<ol>
<li></li>
</ol>
<pre><code> foo
</code></pre>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_ib_i3_ol_nl_i6_t_fb():
    """
    Test case:  Indent of 3 ordered list newline indent of 6 text (indented block)
    was:        test_list_blocks_256ixb
    """

    # Arrange
    source_markdown = """   1.
      foo
"""
    expected_tokens = [
        "[olist(1,4):.:1:6:   :      ]",
        "[BLANK(1,6):]",
        "[para(2,7):]",
        "[text(2,7):foo:]",
        "[end-para:::True]",
        "[BLANK(3,1):]",
        "[end-olist:::True]",
    ]
    expected_gfm = """<ol>
<li>foo</li>
</ol>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_ib_i3_ol_t_nl_i4_t_ib():
    """
    Test case:  Indent of 3 ordered list text newline indent of 4 text (indented block)
    was:        test_list_blocks_256ia
    """

    # Arrange
    source_markdown = """   1. abc
    foo
"""
    expected_tokens = [
        "[olist(1,4):.:1:6:   :    ]",
        "[para(1,7):\n]",
        "[text(1,7):abc\nfoo::\n]",
        "[end-para:::True]",
        "[BLANK(3,1):]",
        "[end-olist:::True]",
    ]
    expected_gfm = """<ol>
<li>abc
foo</li>
</ol>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_ib_i3_ol_t_nl_i10_t_all_ib():
    """
    Test case:  Indent of 3 ordered list text newline indent of 4 text (indented block)
    """

    # Arrange
    source_markdown = """   1. abc
          foo
"""
    expected_tokens = [
        "[olist(1,4):.:1:6:   :      ]",
        "[para(1,7):\n    ]",
        "[text(1,7):abc\nfoo::\n]",
        "[end-para:::True]",
        "[BLANK(3,1):]",
        "[end-olist:::True]",
    ]
    expected_gfm = """<ol>
<li>abc
foo</li>
</ol>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_ib_i3_ol_t_nl_nl_i10_t_all_ib():
    """
    Test case:  Indent of 3 ordered list text newline indent of 4 text (indented block)
    """

    # Arrange
    source_markdown = """   1. abc

          foo
"""
    expected_tokens = [
        "[olist(1,4):.:1:6:   :      ]",
        "[para(1,7):]",
        "[text(1,7):abc:]",
        "[end-para:::True]",
        "[BLANK(2,1):]",
        "[icode-block(3,11):    :]",
        "[text(3,11):foo:]",
        "[end-icode-block:::True]",
        "[BLANK(4,1):]",
        "[end-olist:::True]",
    ]
    expected_gfm = """<ol>
<li>
<p>abc</p>
<pre><code>foo
</code></pre>
</li>
</ol>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_ib_i3_ol_t_nl_i5_t_ib():
    """
    Test case:  Indent of 3 ordered list text newline indent of 5 text (indented block)
    was:        test_list_blocks_256iaa
    """

    # Arrange
    source_markdown = """   1. abc
     foo
"""
    expected_tokens = [
        "[olist(1,4):.:1:6:   :     ]",
        "[para(1,7):\n]",
        "[text(1,7):abc\nfoo::\n]",
        "[end-para:::True]",
        "[BLANK(3,1):]",
        "[end-olist:::True]",
    ]
    expected_gfm = """<ol>
<li>abc
foo</li>
</ol>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_ib_i3_ol_t_nl_i6_t_ib():
    """
    Test case:  Indent of 3 ordered list text newline indent of 6 text (indented block)
    was:        test_list_blocks_256iab
    """

    # Arrange
    source_markdown = """   1. abc
      foo
"""
    expected_tokens = [
        "[olist(1,4):.:1:6:   :      ]",
        "[para(1,7):\n]",
        "[text(1,7):abc\nfoo::\n]",
        "[end-para:::True]",
        "[BLANK(3,1):]",
        "[end-olist:::True]",
    ]
    expected_gfm = """<ol>
<li>abc
foo</li>
</ol>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_ib_ol_ol_nl_i4_t_ib():
    """
    Test case:  Ordered list x2 newline indent of 4 text (indented block)
    """

    # Arrange
    source_markdown = """1. 1.
    foo
"""
    expected_tokens = [
        "[olist(1,1):.:1:3::   ]",
        "[olist(1,4):.:1:6:   ]",
        "[BLANK(1,6):]",
        "[end-olist:::True]",
        "[para(2,5): ]",
        "[text(2,5):foo:]",
        "[end-para:::True]",
        "[BLANK(3,1):]",
        "[end-olist:::True]",
    ]
    expected_gfm = """<ol>
<li>
<ol>
<li></li>
</ol>
foo</li>
</ol>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_ib_ol_ol_nl_i10_t_all_ib():
    """
    Test case:  Ordered list x2 newline indent of 4 text (indented block)
    """

    # Arrange
    source_markdown = """1. 1.
          foo
"""
    expected_tokens = [
        "[olist(1,1):.:1:3:]",
        "[olist(1,4):.:1:6:   :      ]",
        "[BLANK(1,6):]",
        "[icode-block(2,11):    :]",
        "[text(2,11):foo:]",
        "[end-icode-block:::True]",
        "[BLANK(3,1):]",
        "[end-olist:::True]",
        "[end-olist:::True]",
    ]
    expected_gfm = """<ol>
<li>
<ol>
<li>
<pre><code>foo
</code></pre>
</li>
</ol>
</li>
</ol>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_ib_ol_ol_t_nl_i4_t_ib():
    """
    Test case:  Ordered list x2 text newline indent of 4 text (indented block)
    was:        test_list_blocks_256ib
    """

    # Arrange
    source_markdown = """1. 1. abc
    foo
"""
    expected_tokens = [
        "[olist(1,1):.:1:3:]",
        "[olist(1,4):.:1:6:   :    ]",
        "[para(1,7):\n]",
        "[text(1,7):abc\nfoo::\n]",
        "[end-para:::True]",
        "[BLANK(3,1):]",
        "[end-olist:::True]",
        "[end-olist:::True]",
    ]
    expected_gfm = """<ol>
<li>
<ol>
<li>abc
foo</li>
</ol>
</li>
</ol>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_ib_ol_ol_t_nl_i4_t_all_ib():
    """
    Test case:  Ordered list x2 text newline indent of 4 text (indented block)
    """

    # Arrange
    source_markdown = """1. 1. abc
          foo
"""
    expected_tokens = [
        "[olist(1,1):.:1:3:]",
        "[olist(1,4):.:1:6:   :      ]",
        "[para(1,7):\n    ]",
        "[text(1,7):abc\nfoo::\n]",
        "[end-para:::True]",
        "[BLANK(3,1):]",
        "[end-olist:::True]",
        "[end-olist:::True]",
    ]
    expected_gfm = """<ol>
<li>
<ol>
<li>abc
foo</li>
</ol>
</li>
</ol>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_ib_ol_ol_t_nl_nl_i4_t_all_ib():
    """
    Test case:  Ordered list x2 text newline newline indent of 4 text (indented block)
    """

    # Arrange
    source_markdown = """1. 1. abc

          foo
"""
    expected_tokens = [
        "[olist(1,1):.:1:3:]",
        "[olist(1,4):.:1:6:   :      ]",
        "[para(1,7):]",
        "[text(1,7):abc:]",
        "[end-para:::True]",
        "[BLANK(2,1):]",
        "[icode-block(3,11):    :]",
        "[text(3,11):foo:]",
        "[end-icode-block:::True]",
        "[BLANK(4,1):]",
        "[end-olist:::True]",
        "[end-olist:::True]",
    ]
    expected_gfm = """<ol>
<li>
<ol>
<li>
<p>abc</p>
<pre><code>foo
</code></pre>
</li>
</ol>
</li>
</ol>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_ib_ol_nl_i3_ol_nl_i4_t_ib():
    """
    Test case:  Ordered list newline indent of 3 ordered list newline indent of 4 text (indented block)
    """

    # Arrange
    source_markdown = """1.
   1.
    foo
"""
    expected_tokens = [
        "[olist(1,1):.:1:3::   ]",
        "[BLANK(1,3):]",
        "[olist(2,4):.:1:6:   ]",
        "[BLANK(2,6):]",
        "[end-olist:::True]",
        "[para(3,5): ]",
        "[text(3,5):foo:]",
        "[end-para:::True]",
        "[BLANK(4,1):]",
        "[end-olist:::True]",
    ]
    expected_gfm = """<ol>
<li>
<ol>
<li></li>
</ol>
foo</li>
</ol>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_ib_ol_t_nl_i3_ol_nl_i4_t_ib():
    """
    Test case:  Ordered list text newline indent of 3 ordered list newline indent of 4 text (indented block)
    """

    # Arrange
    source_markdown = """1. abc
   1.
    foo
"""
    expected_tokens = [
        "[olist(1,1):.:1:3::   \n   ]",
        "[para(1,4):\n\n ]",
        "[text(1,4):abc\n1.\nfoo::\n\n]",
        "[end-para:::True]",
        "[BLANK(4,1):]",
        "[end-olist:::True]",
    ]
    expected_gfm = """<ol>
<li>abc
1.
foo</li>
</ol>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_ib_ul_t_nl_i2_ul_nl_i4_t_ib():
    """
    Test case:  Unordered list text newline indent of 2 unordered list newline indent of 4 text (indented block)
    """

    # Arrange
    source_markdown = """- abc
  -
    foo
"""
    expected_tokens = [
        "[ulist(1,1):-::2::  \n  ]",
        "[setext(2,3):-:1::(1,3)]",
        "[text(1,3):abc:]",
        "[end-setext::]",
        "[para(3,5):  ]",
        "[text(3,5):foo:]",
        "[end-para:::True]",
        "[BLANK(4,1):]",
        "[end-ulist:::True]",
    ]
    expected_gfm = """<ul>
<li>
<h2>abc</h2>
foo</li>
</ul>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_ib_ul_t_nl_i2_ulb_nl_i4_t_ib():
    """
    Test case:  Unordered list text newline indent of 2 unordered list (b) newline indent of 4 text (indented block)
    """

    # Arrange
    source_markdown = """- abc
  *
    foo
"""
    expected_tokens = [
        "[ulist(1,1):-::2::  \n  ]",
        "[para(1,3):\n\n  ]",
        "[text(1,3):abc\n::\n]",
        "[text(2,1):*:]",
        "[text(2,2):\nfoo::\n]",
        "[end-para:::True]",
        "[BLANK(4,1):]",
        "[end-ulist:::True]",
    ]
    expected_gfm = """<ul>
<li>abc
*
foo</li>
</ul>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_ib_ol_nl_i3_ol_t_nl_i4_t_ib():
    """
    Test case:  Ordered list newline indent of 3 ordered list text newline indent of 4 text (indented block)
    was:        test_list_blocks_256ic
    """

    # Arrange
    source_markdown = """1.
   1. abc
    foo
"""
    expected_tokens = [
        "[olist(1,1):.:1:3:]",
        "[BLANK(1,3):]",
        "[olist(2,4):.:1:6:   :    ]",
        "[para(2,7):\n]",
        "[text(2,7):abc\nfoo::\n]",
        "[end-para:::True]",
        "[BLANK(4,1):]",
        "[end-olist:::True]",
        "[end-olist:::True]",
    ]
    expected_gfm = """<ol>
<li>
<ol>
<li>abc
foo</li>
</ol>
</li>
</ol>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_ib_ol_t_nl_i3_ol_t_nl_i4_t_ib():
    """
    Test case:  Ordered list text newline indent of 3 ordered list text newline indent of 4 text (indented block)
    """

    # Arrange
    source_markdown = """1. abc
   1. abc
    foo
"""
    expected_tokens = [
        "[olist(1,1):.:1:3:]",
        "[para(1,4):]",
        "[text(1,4):abc:]",
        "[end-para:::True]",
        "[olist(2,4):.:1:6:   :    ]",
        "[para(2,7):\n]",
        "[text(2,7):abc\nfoo::\n]",
        "[end-para:::True]",
        "[BLANK(4,1):]",
        "[end-olist:::True]",
        "[end-olist:::True]",
    ]
    expected_gfm = """<ol>
<li>abc
<ol>
<li>abc
foo</li>
</ol>
</li>
</ol>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_ib_i3_ol_nl_i5_ol_nl_i4_t_ib():
    """
    Test case:  Indent of 3 ordered list newline indent of 5 ordered list newline indent of 4 text (indented block)
    """

    # Arrange
    source_markdown = """   1.
      1. abc
    foo
"""
    expected_tokens = [
        "[olist(1,4):.:1:6:   ]",
        "[BLANK(1,6):]",
        "[olist(2,7):.:1:9:      :    ]",
        "[para(2,10):\n]",
        "[text(2,10):abc\nfoo::\n]",
        "[end-para:::True]",
        "[BLANK(4,1):]",
        "[end-olist:::True]",
        "[end-olist:::True]",
    ]
    expected_gfm = """<ol>
<li>
<ol>
<li>abc
foo</li>
</ol>
</li>
</ol>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_ib_i3_ol_t_nl_i5_ol_nl_i4_t_ib():
    """
    Test case:  Indent of 3 ordered list text newline indent of 5 ordered list newline indent of 4 text (indented block)
    """

    # Arrange
    source_markdown = """   1. abc
      1. abc
    foo
"""
    expected_tokens = [
        "[olist(1,4):.:1:6:   ]",
        "[para(1,7):]",
        "[text(1,7):abc:]",
        "[end-para:::True]",
        "[olist(2,7):.:1:9:      :    ]",
        "[para(2,10):\n]",
        "[text(2,10):abc\nfoo::\n]",
        "[end-para:::True]",
        "[BLANK(4,1):]",
        "[end-olist:::True]",
        "[end-olist:::True]",
    ]
    expected_gfm = """<ol>
<li>abc
<ol>
<li>abc
foo</li>
</ol>
</li>
</ol>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_ib_i3_ol_nl_i5_ol_t_nl_i4__t_ib():
    """
    Test case:  Indent of 3 ordered list newline indent of 5 ordered list text newline indent of 4 text (indented block)
    was:        test_list_blocks_256id
    """

    # Arrange
    source_markdown = """   1.
      1. abc
    foo
"""
    expected_tokens = [
        "[olist(1,4):.:1:6:   ]",
        "[BLANK(1,6):]",
        "[olist(2,7):.:1:9:      :    ]",
        "[para(2,10):\n]",
        "[text(2,10):abc\nfoo::\n]",
        "[end-para:::True]",
        "[BLANK(4,1):]",
        "[end-olist:::True]",
        "[end-olist:::True]",
    ]
    expected_gfm = """<ol>
<li>
<ol>
<li>abc
foo</li>
</ol>
</li>
</ol>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_ib_i3_ol_t_nl_i5_ol_t_nl_i4_t_ib():
    """
    Test case:  Indent of 3 ordered list text newline indent of 5 ordered list text newline indent of 4 text (indented block)
    """

    # Arrange
    source_markdown = """   1. abc
      1. abc
    foo
"""
    expected_tokens = [
        "[olist(1,4):.:1:6:   ]",
        "[para(1,7):]",
        "[text(1,7):abc:]",
        "[end-para:::True]",
        "[olist(2,7):.:1:9:      :    ]",
        "[para(2,10):\n]",
        "[text(2,10):abc\nfoo::\n]",
        "[end-para:::True]",
        "[BLANK(4,1):]",
        "[end-olist:::True]",
        "[end-olist:::True]",
    ]
    expected_gfm = """<ol>
<li>abc
<ol>
<li>abc
foo</li>
</ol>
</li>
</ol>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_ib_i1_ol_nl_i4_ol_nl_i4_t_ib():
    """
    Test case:  Indent of 1 ordered list newline indent of 4 ordered list newline indent of 4 text (indented block)
    """

    # Arrange
    source_markdown = """ 1. abc
    1.
    foo
"""
    expected_tokens = [
        "[olist(1,2):.:1:4: :    \n    ]",
        "[para(1,5):\n\n]",
        "[text(1,5):abc\n1.\nfoo::\n\n]",
        "[end-para:::True]",
        "[BLANK(4,1):]",
        "[end-olist:::True]",
    ]
    expected_gfm = """<ol>
<li>abc
1.
foo</li>
</ol>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_ib_i2_ul_nl_i4_ul_nl_i4_t_ib():
    """
    Test case:  Indent of 2 unordered list newline indent of 4 unordered list newline indent of 4 text (indented block)
    """

    # Arrange
    source_markdown = """  - abc
    -
    foo
"""
    expected_tokens = [
        "[ulist(1,3):-::4:  :    \n    ]",
        "[setext(2,5):-:1::(1,5)]",
        "[text(1,5):abc:]",
        "[end-setext::]",
        "[para(3,5):]",
        "[text(3,5):foo:]",
        "[end-para:::True]",
        "[BLANK(4,1):]",
        "[end-ulist:::True]",
    ]
    expected_gfm = """<ul>
<li>
<h2>abc</h2>
foo</li>
</ul>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_ib_i2_ul_nl_i4_ulb_nl_i4_t_ib():
    """
    Test case:  Indent of 2 unordered list newline indent of 4 unordered list (b) newline indent of 4 text (indented block)
    """

    # Arrange
    source_markdown = """  - abc
    *
    foo
"""
    expected_tokens = [
        "[ulist(1,3):-::4:  :    \n    ]",
        "[para(1,5):\n\n]",
        "[text(1,5):abc\n::\n]",
        "[text(2,1):*:]",
        "[text(2,2):\nfoo::\n]",
        "[end-para:::True]",
        "[BLANK(4,1):]",
        "[end-ulist:::True]",
    ]
    expected_gfm = """<ul>
<li>abc
*
foo</li>
</ul>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_ib_i1_ol_t_nl_i4_ol_nl_i4_t_ib():
    """
    Test case:  Indent of 1 ordered list text newline indent of 4 ordered list newline indent of 4 text (indented block)
    """

    # Arrange
    source_markdown = """ 1. abc
    1.
    foo
"""
    expected_tokens = [
        "[olist(1,2):.:1:4: :    \n    ]",
        "[para(1,5):\n\n]",
        "[text(1,5):abc\n1.\nfoo::\n\n]",
        "[end-para:::True]",
        "[BLANK(4,1):]",
        "[end-olist:::True]",
    ]
    expected_gfm = """<ol>
<li>abc
1.
foo</li>
</ol>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_ib_i2_ul_t_nl_i4_ul_nl_i4_t_ib():
    """
    Test case:  Indent of 2 unordered list text newline indent of 4 unordered list newline indent of 4 text (indented block)
    """

    # Arrange
    source_markdown = """  - abc
    -
    foo
"""
    expected_tokens = [
        "[ulist(1,3):-::4:  :    \n    ]",
        "[setext(2,5):-:1::(1,5)]",
        "[text(1,5):abc:]",
        "[end-setext::]",
        "[para(3,5):]",
        "[text(3,5):foo:]",
        "[end-para:::True]",
        "[BLANK(4,1):]",
        "[end-ulist:::True]",
    ]
    expected_gfm = """<ul>
<li>
<h2>abc</h2>
foo</li>
</ul>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_ib_i2_ul_t_nl_i4_ulb_nl_i4_t_ib():
    """
    Test case:  Indent of 2 unordered list text newline indent of 4 unordered list (b) newline indent of 4 text (indented block)
    """

    # Arrange
    source_markdown = """  - abc
    *
    foo
"""
    expected_tokens = [
        "[ulist(1,3):-::4:  :    \n    ]",
        "[para(1,5):\n\n]",
        "[text(1,5):abc\n::\n]",
        "[text(2,1):*:]",
        "[text(2,2):\nfoo::\n]",
        "[end-para:::True]",
        "[BLANK(4,1):]",
        "[end-ulist:::True]",
    ]
    expected_gfm = """<ul>
<li>abc
*
foo</li>
</ul>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_ib_i1_ol_nl_i4_ol_t_nl_i4_t_ib():
    """
    Test case:  Indent of 1 ordered list newline indent of 4 ordered list text newline indent of 4 text (indented block)
    """

    # Arrange
    source_markdown = """ 1.
    1. def
    foo
"""
    expected_tokens = [
        "[olist(1,2):.:1:4: ]",
        "[BLANK(1,4):]",
        "[olist(2,5):.:1:7:    :    ]",
        "[para(2,8):\n]",
        "[text(2,8):def\nfoo::\n]",
        "[end-para:::True]",
        "[BLANK(4,1):]",
        "[end-olist:::True]",
        "[end-olist:::True]",
    ]
    expected_gfm = """<ol>
<li>
<ol>
<li>def
foo</li>
</ol>
</li>
</ol>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_m_ib_i1_ol_t_nl_i4_ol_t_nl_i4_t_ib():
    """
    Test case:  Indent of 1 ordered list text newline indent of 4 ordered list text newline indent of 4 text (indented block)
    was:        test_list_blocks_256ie
    """

    # Arrange
    source_markdown = """ 1. abc
    1. def
    foo
"""
    expected_tokens = [
        "[olist(1,2):.:1:4: ]",
        "[para(1,5):]",
        "[text(1,5):abc:]",
        "[end-para:::True]",
        "[olist(2,5):.:1:7:    :    ]",
        "[para(2,8):\n]",
        "[text(2,8):def\nfoo::\n]",
        "[end-para:::True]",
        "[BLANK(4,1):]",
        "[end-olist:::True]",
        "[end-olist:::True]",
    ]
    expected_gfm = """<ol>
<li>abc
<ol>
<li>def
foo</li>
</ol>
</li>
</ol>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)



"""
https://github.github.com/gfm/#paragraph
"""
import pytest

from .utils import act_and_assert


@pytest.mark.gfm
def test_paragraph_series_b_b():
    """
    Test case:  Paragraph containing a backslash
    was:        test_paragraph_extra_12
    """

    # Arrange
    source_markdown = """a \\\\fun\\\\ day"""
    expected_tokens = [
        "[para(1,1):]",
        "[text(1,1):a \\\b\\fun\\\b\\ day:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>a \\fun\\ day</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_b_cs():
    """
    Test case:  Paragraph containing a code span.
    was:        test_paragraph_extra_13
    """

    # Arrange
    source_markdown = """a ``fun`` day"""
    expected_tokens = [
        "[para(1,1):]",
        "[text(1,1):a :]",
        "[icode-span(1,3):fun:``::]",
        "[text(1,10): day:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>a <code>fun</code> day</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_b_cr():
    """
    Test case:  Paragraph containing a character reference.
    was:        test_paragraph_extra_14
    """

    # Arrange
    source_markdown = """fun &amp; joy"""
    expected_tokens = [
        "[para(1,1):]",
        "[text(1,1):fun \a&amp;\a\a&\a&amp;\a\a joy:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>fun &amp; joy</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_b_rh():
    """
    Test case:  Paragraph containing a raw html block.
    was:        test_paragraph_extra_15
    """

    # Arrange
    source_markdown = """where <there it='is'> it"""
    expected_tokens = [
        "[para(1,1):]",
        "[text(1,1):where :]",
        "[raw-html(1,7):there it='is']",
        "[text(1,22): it:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>where <there it='is'> it</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_b_ua():
    """
    Test case:  Paragraph containing an URI autolink
    was:        test_paragraph_extra_16
    """

    # Arrange
    source_markdown = """look <http://www.google.com> for"""
    expected_tokens = [
        "[para(1,1):]",
        "[text(1,1):look :]",
        "[uri-autolink(1,6):http://www.google.com]",
        "[text(1,29): for:]",
        "[end-para:::True]",
    ]
    expected_gfm = (
        """<p>look <a href="http://www.google.com">http://www.google.com</a> for</p>"""
    )

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_b_ea():
    """
    Test case:  Paragraph containing an email autolink
    was:        test_paragraph_extra_17
    """

    # Arrange
    source_markdown = """email <foo@bar.com> for"""
    expected_tokens = [
        "[para(1,1):]",
        "[text(1,1):email :]",
        "[email-autolink(1,7):foo@bar.com]",
        "[text(1,20): for:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>email <a href="mailto:foo@bar.com">foo@bar.com</a> for</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_b_e():
    """
    Test case:  Paragraph containing emphasis
    was:        test_paragraph_extra_18
    """

    # Arrange
    source_markdown = """really! *it's me!* here!"""
    expected_tokens = [
        "[para(1,1):]",
        "[text(1,1):really! :]",
        "[emphasis(1,9):1:*]",
        "[text(1,10):it's me!:]",
        "[end-emphasis(1,18)::]",
        "[text(1,19): here!:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>really! <em>it's me!</em> here!</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_b_l():
    """
    Test case:  Paragraph containing a link.
    was:        test_paragraph_extra_19
    """

    # Arrange
    source_markdown = """at [Foo](/uri "t") more"""
    expected_tokens = [
        "[para(1,1):]",
        "[text(1,1):at :]",
        '[link(1,4):inline:/uri:t::::Foo:False:":: :]',
        "[text(1,5):Foo:]",
        "[end-link::]",
        "[text(1,19): more:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>at <a href="/uri" title="t">Foo</a> more</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)


@pytest.mark.gfm
def test_paragraph_series_b_i():
    """
    Test case:  Paragraph containing an image
    was:        test_paragraph_extra_20
    """

    # Arrange
    source_markdown = """my ![foo](/url "t") image"""
    expected_tokens = [
        "[para(1,1):]",
        "[text(1,1):my :]",
        '[image(1,4):inline:/url:t:foo::::foo:False:":: :]',
        "[text(1,20): image:]",
        "[end-para:::True]",
    ]
    expected_gfm = """<p>my <img src="/url" alt="foo" title="t" /> image</p>"""

    # Act & Assert
    act_and_assert(source_markdown, expected_gfm, expected_tokens)



"""
Module to implement a sample plugin that just reports that it has been called.
"""
from pymarkdown.plugin_manager import Plugin, PluginDetails


class PluginOne(Plugin):
    """
    Class to implement a sample plugin that just reports that it has been called.
    """

    __valid_values = [0, 1, 2]

    def __init__(self):
        super().__init__()
        self.test_value = None
        self.other_test_value = None

    def get_details(self):
        """
        Get the details for the plugin.
        """
        return PluginDetails(
            plugin_name="debug-only",
            plugin_id="MD999",
            plugin_enabled_by_default=False,
            plugin_description="Debug plugin",
            plugin_version="0.0.0",
            plugin_interface_version=1,
        )

    @classmethod
    def __validate_configuration_other_test_value(cls, found_value):
        if found_value not in PluginOne.__valid_values:
            raise ValueError(f"Allowable values: {str(found_value)}")

    def initialize_from_config(self):
        """
        Event to allow the plugin to load configuration.
        """
        print(self.get_details().plugin_id + ">>init_from_config")
        self.test_value = self.plugin_configuration.get_integer_property(
            "test_value", default_value=1
        )
        print(self.get_details().plugin_id + ">>test_value>>" + str(self.test_value))
        self.other_test_value = self.plugin_configuration.get_integer_property(
            "other_test_value",
            default_value=1,
            valid_value_fn=self.__validate_configuration_other_test_value,
        )
        print(
            self.get_details().plugin_id
            + ">>other_test_value>>"
            + str(self.other_test_value)
        )
        if self.test_value == 10:
            raise Exception("because")

    def starting_new_file(self):
        """
        Event that the a new file to be scanned is starting.
        """
        print(self.get_details().plugin_id + ">>starting_new_file>>")

    def next_line(self, context, line):
        """
        Event that a new line is being processed.
        """
        print(self.get_details().plugin_id + ">>next_line:" + line)

    def next_token(self, context, token):
        """
        Event that a new token is being processed.
        """
        print(self.get_details().plugin_id + ">>token:" + str(token))
        if self.test_value == 20:
            raise Exception("because")

    def completed_file(self, context):
        """
        Event that the file being currently scanned is now completed.
        """
        print(self.get_details().plugin_id + ">>completed_file")
